import java.util.*;

/* Sym.java
 * Author: John Lenz <jelenz@wisc.edu>
 * (C) Copyright 2003 John Lenz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

class Type {
  public final String name;
  private  String _structName;
  private int _numPointers;
  
  /// Private so Type acts like an enumeration (can only be accessed through static fields).
  private Type(String n) { 
    name = n; 
    _numPointers = 0;
    _structName = "";
  }

  public String toString() { 
    if (name.equals("struct") && _structName != "")
      return _structName;
    else
      return name; 
  }

  public boolean equals(Type a) {
    if (name.equals(a.name)) {
      if (name == "struct" && (_structName == "" || a._structName == ""))
        return _numPointers == a._numPointers;
      else
        return _numPointers == a._numPointers && _structName.equals(a._structName);
    } else {
      return false;
    }
  }

  public Type addPointer(int num) {
    Type t = new Type(name);
    t._numPointers = _numPointers + num;
    t._structName = _structName;
    return t;
  }

  public int numPointers() {
    return _numPointers;
  }

  public static final Type Error = new Type("error");
  public static final Type Func = new Type("function");
  public static final Type Int = new Type("int");
  public static final Type Bool = new Type("bool");
  public static final Type Void = new Type("void");
  public static final Type String = new Type("string");
  public static final Type NULL = new Type("NULL");
  public static final Type Struct = new Type("struct");

  public static Type Struct(String s) {
    Type t = new Type("struct");
    t._structName = s;
    return t;
  }
}

public class Sym {
  /// Initialize sym as a normal type.
  /// arraySize will be VarDeclNode.NOT_ARRAY = -1 if not an array
  public Sym(String name, Type type, int arraySize, int off) {
    _name = name;
    _type = type;
    _paramList = null;
    _arraySize = arraySize;
    _offset = off;
  }

  public Sym(String name, Type type, int off) {
    _name = name;
    _type = type;
    _paramList = null;
    _arraySize = -1;
    _offset = off;
  }
  
  /// Initialize sym as a function
  public Sym(String name, LinkedList list) {
    _name = name;
    _type = Type.Func;
    _paramList = list;
    _offset = -1;
  }

  // return the name of the symbol.
  public String name() {
    return _name;
  }

  // return the type.
  public Type type() {
    return _type;
  }

  // return the param list or null
  public LinkedList params() {
    return _paramList;
  }

  public boolean isArray() {
    return _arraySize >= 0;
  }

  public int size() {
    return _arraySize;
  }

  public void setSize(int s) {
    _arraySize = s;
  }

  public String toString() {
    if (_type.numPointers() > 0)
      return _name + "(" + _type + "[" + _arraySize + "," + _type.numPointers() + "]<" + _offset + ">)";
    else
      return _name + "(" + _type + "<" + _offset + ">)";
  }

  public int offset() {
    return _offset;
  }

  private String _name;
  private Type _type;
  /// list of function paramaters.  Is null if symbol is not a function.
  /// The first entry in the list is the return type.
  private LinkedList _paramList;
  /// Array size.  Is -1 if symbol is not an array.
  private int _arraySize;
  private int _offset;
}
