import java.io.*;
import java.util.*;

/* SymTab.java
 * Author: John Lenz <jelenz@wisc.edu>
 * (C) Copyright 2003 John Lenz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

class DuplicateException extends Exception { }
class EmptySymTabException extends Exception { }

public class SymTab {

  // Initialize the SymTab to contain one hashtable
  public SymTab() {
    _list = new LinkedList();
    addHashtab();
  }

  /* Insert the Sym into the top hashtable.
   * Throws EmptySymTabException if the list is empty.
   * Throws DuplicateException if sym is already in the hashtable.
   */
  public void insert(Sym sym) throws DuplicateException, EmptySymTabException {
    if (_list.isEmpty())
      throw new EmptySymTabException();

    Hashtable h = (Hashtable) _list.getFirst();
    if (h.containsKey(sym.name()))
      throw new DuplicateException();
    h.put(sym.name(), sym);
  }

  // Add a new hashtable to the front of the list
  public void addHashtab() {
    _list.addFirst(new Hashtable());
  }

  // If the first hashtable contains name, return it.  Otherwise, return null.
  public Sym localLookup(String name) {
    if (_list.isEmpty())
      return null;
    Hashtable h = (Hashtable) _list.getFirst();
    return (Sym) h.get(name);
  }

  // If any hashtable contains name, return it.  Otherwise, return null.
  public Sym globalLookup(String name) {
    Iterator i = _list.iterator();

    while (i.hasNext()) {
      Hashtable h = (Hashtable) i.next();
      Sym s = (Sym) h.get(name);
      if (s != null)
        return s;
    }
    return null;
  }

  // Remove the first hashtable.  Throws EmptySymTabException if list is empty.
  public void removeHashtab() throws EmptySymTabException {
    if (_list.isEmpty())
      throw new EmptySymTabException();
    _list.removeFirst();
  }

  // Print out all the hashtables.
  public void print(PrintWriter p) {
    Iterator i = _list.iterator();
    while (i.hasNext()) {
      Hashtable h = (Hashtable) i.next();
      p.print(h.toString());
      p.print("\n");
    }
  }

  private LinkedList _list;
}
