/* cmm.java
 * Author: John Lenz <jelenz@wisc.edu>
 * (C) Copyright 2003 John Lenz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.io.*;
import java_cup.runtime.*;

// **********************************************************************
// Main program to test the C-- parser.
//
// The program opens the two files, creates a scanner and a parser, and
// calls the parser.  If the parse is successful, the AST is unparsed.
// **********************************************************************

public class cmm {
    public static void main(String[] args)
        throws IOException // may be thrown by the scanner
    {
      String inName="", outName="";

      // check for command-line args
      if (args.length == 3) {
        if (args[0].equals("--mips")) {
          Codegen.Wisc = false;
          Codegen.wordSize = 4;
          Codegen.ADD = "addu";
          Codegen.SUB = "subu";
          inName = args[1];
          outName = args[2];
        } else {
          System.err.println("usage: cmm [--mips] <input file> <output file>");
          System.exit(-1);
        }
      } else {
        if (args.length == 2) {
          inName = args[0];
          outName = args[1];
        } else {
          System.err.println("usage: cmm [--mips] <input file> <output file>");
          System.exit(-1);
        }
      }

        // open input file
        FileReader inFile = null;
        try {
            inFile = new FileReader(inName);
        } catch (FileNotFoundException ex) {
            System.err.println("File " + inName + " not found.");
            System.exit(-1);
        }

        parser P = new parser(new Yylex(inFile));

        ProgramNode root=null; // the parser will return a Symbol whose value
                          // field's type is the type associated with the
                          // root nonterminal (i.e., with the nonterminal
                          // "program")

        try {
            root = (ProgramNode) P.parse().value; // do the parse
        } catch (Exception ex){
            System.err.println("Exception occured during parse: " + ex);
            System.exit(-1);
        }
        
        symTable = new SymTab();
        root.updateNames(symTable);

        if (Errors.fatalError) {
          System.err.println("Confused by earlier errors: aborting");
          System.exit(0);
        }

        // only open output file if everything else is ok.
        // open output file
        /*PrintWriter unparse = null;
        try {
            unparse = IO.openOutputFile(inName + ".unparse");
        } catch (IOException ex) {
            System.err.println("File " + inName + ".unparse could not be opened.");
            System.exit(-1);
        }

        root.unparse(unparse, 0);
        unparse.close();*/

        root.checkTypes();

        if (Errors.fatalError) {
          System.err.println("Confused by earlier errors: aborting");
          System.exit(0);
        }

        // only open output file if everything else is ok.
        // open output file
        PrintWriter outFile = null;
        try {
            outFile = IO.openOutputFile(outName);
        } catch (IOException ex) {
            System.err.println("File " + outName + " could not be opened.");
            System.exit(-1);
        }

        Codegen.p = outFile;
        root.codeGen();

        if (Codegen.Wisc) {
          WiscPrint.export(outFile);
        }

        outFile.close();

        return;
    }

    public static SymTab symTable;
}
