int randSeed;
int randA;
int randC;
int randM;

int rand() {
  randSeed = (randSeed * randA + randC) % randM;
  return randSeed;
}

struct maze_loc {
  int rank;
  int x;
  int y;
  struct maze_loc *ptr;
};

struct hedge_loc {
  int x;
  int y;
  int pos;
  int on;
};

struct maze_loc *maze;
struct hedge_loc *hedge;
struct hedge_loc **hedge_list;

void link(struct maze_loc *a, struct maze_loc *b) {
  if (a->rank == b->rank) {
    a->ptr = b;
    b->rank += 1;
    return;
  }
  if (a->rank > b->rank) {
    b->ptr = a;
    return;
  }
  a->ptr = b;
}

struct maze_loc *find(struct maze_loc *a) {
  if (a != a->ptr) {
    a->ptr = find(a->ptr);
  }
  return a->ptr;
}

// the maze is generated by making a list of all the internal hedges
// and randomizing it, then going lineraly through the list, we take a
// hedge and see if the maze squares adjacent to it are already connected
// (with find) is not the we connect them (with link), this prevents us
// from creating a maze with a cycle because we will not link two maze
// squares that are already connect by some path

void main() {
  int x; int y; int i; int j; int k; int l; int tmp;
  struct maze_loc *a; struct maze_loc *b;
  struct hedge_loc *htmp;

  randA = 151;
  randC = 233;
  randM = 179;
  randSeed = 139;

  x = 15;
  y = 15;

  maze = sbrk(sizeof(maze_loc)*x*y);
  hedge = sbrk(sizeof(hedge_loc)*((x*(y-1))+((x-1)*y)));
  hedge_list = sbrk(4 * ((x*(y-1))+((x-1)*y)));

  // init maze
  for (j = 0; j < y; j += 1) {
    for (i = 0; i < x; i += 1) {
      maze[x*j+i].x = i;
      maze[x*j+i].y = j;
      maze[x*j+i].ptr = &maze[x*j+i];
      maze[x*j+i].rank = 0;
    }
  }

  // init hedges
  for (j = 0; j < y - 1; j += 1) {
    for (i = 0; i < x; i += 1) {
      hedge[x*j+i].x = i;
      hedge[x*j+i].y = j;
      hedge[x*j+i].pos = 1;
      hedge[x*j+i].on = 1;
      hedge_list[x*j+i] = &hedge[x*j+i];
    }
  }
  k = x * (y - 1);

  for (j = 0; j < y; j += 1) {
    for (i = 0; i < x - 1; i += 1) {
      hedge[k+(x-1)*j+i].x = i;
      hedge[k+(x-1)*j+i].y = j;
      hedge[k+(x-1)*j+i].pos = 2;
      hedge[k+(x-1)*j+i].on = 1;
      hedge_list[k+(x-1)*j+i] = &hedge[k+(x-1)*j+i];
    }
  }

  k = x*(y-1) + (x-1)*y;

  // randomize hedges
  for (i = k - 1; i > 0; i -= 1) {
    htmp = hedge_list[i];
    j = rand() % i;
    hedge_list[i] = hedge_list[j];
    hedge_list[j] = htmp;
  }

  l = k;

  // create maze
  for (i = 0; i < l; i += 1) {
    j = hedge_list[i]->x;
    k = hedge_list[i]->y;
    a = find(&maze[x*k+j]);
    if (hedge_list[i]->pos == 1) {
      b = find(&maze[x*(k+1)+j]);
    } else {
      b = find(&maze[x*k+j+1]);
    }
    if (a != b) {
      link(a, b);
      hedge_list[i]->on = 0;
    }
  }

  // print out the maze
  
  cout << "+";
  for (i = 0; i < x; i += 1) {
    cout << "-+";
  }
  cout << "\n";

  l = x*(y-1);

  for (j = 0; j < y; j += 1) {
    cout << "|";
    for (i = 0; i < x - 1; i += 1) {
      if (hedge[l + (x-1)*j+i].on == 1) {
        cout << " |";
      } else {
        cout << "  ";
      }
    }
    cout << " |\n|";
    if (j < y - 1) {
      for (i = 0; i < x; i += 1) {
        if (hedge[j*x+i].on == 1) {
          cout << "--";
        } else {
          cout << " -";
        }
      }
      cout << "\n";
    }
  }

  for (i = 0; i < x; i += 1) {
    cout << "-+";
  }
  cout << "\n";
}
